//*********************************************************************************
//*                                                                               *
//*         Name:  keypad.c                                                       *
//*                                                                               *
//*       Author:  Daniel F. Ramirez                                              *
//*                                                                               *
//*         Date:  08/07/2009                                                     *
//*                                                                               *
//*  Description:                                                                 *
//*                                                                               *
//*   	This is a PIC 18C8520 C test program which will test the Vex Keypad       *
//*  functions.                                                                   *
//*                                                                               *
//*                                                                               *
//*     Future upgrades to this application include: Implementing the Stamp       *
//* BS2 NCD functions.                                                            *
//*                                                                               *
//*********************************************************************************

  
// Include file headers here
/*
#include "flags.h"

#ifdef DSPIC30F2010     
#include <p30F2010.h>					// Register definitions for DSPIC30F2010
#endif

#ifdef DSPIC30F6014     
#include <p30F6014.h>					// Register definitions for DSPIC30F6014
#endif
 
#include <stdio.h>
#include <dsp.h>
#include <ports.h>						// Include support for ports
#include <uart.h> 						// UART library functions
#include <math.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <timer.h>                                     // Timer library functions                                                                 
//#include <delays.h>                                     // High resolution delay functions
#include "timeutil.h"                                   // Timer library functions                               
//#include "portutil.h"                                   // My own I/O Port library functions
#include "uartutil.h"                          // UART support functions
//#include "lcd.h"						// LCD Support using SPI2 
#include "portutil.h"                                   // port utilities 

*/
#include "BuiltIns.h"
#include "ifi_default.h"
#include "ifi_aliases.h"

#include "ifi_default.h"
#include "my_types.h"					// General C/C++ types
//#include "timeutil.h"                                   // Timer utilities
#include "keypad.h"                                     // Keypad function headers
//#include "portutil.h"

//********************************************************************************
// Constant Definitions
//********************************************************************************
 
#define KEYPAD_DRIVER                                    // Include Calculator Keypad

//#define NCD_TEST                                      // Select the BS2 NCD  function test
#define LOW  0                                  // Logic low
#define HIGH 1                                  // Logic high

 
 
 

//********************************************************************************
// Keypad variable declarations
//********************************************************************************


// Keypad general variables


/* Declare global keypad variables here */

  

int             db;                                        // Debounce bit used by KeyScan
int             press;                                     // Flag to indicate keypress
int             key;                                       // Key number 0-15.
byte            row;                                       // Counter used in scanning keys.
byte            col;
byte            cols;                                      // Input states of pins PORTH.4-PORTH.7.

int             done = FALSE;
char            c;



const int           Cols[4] = {0xFE, 0xFD, 0xFB, 0xF7};
//const int            Cols[4] = {0xEF, 0xDF, 0xBF, 0x7F};

const int            KeyLookupTable[16] = {0xD7, 0xEE, 0xDE, 0xBE, 0xED, 0xDD,
                                   0xBD, 0xEB, 0xDB, 0xBB, 0x7E, 0x7D,
                                   0x7B, 0x77, 0xE7, 0xb7};

const byte            Table[8] = {1, 2, 4, 8, 16, 32, 64, 128};   // Table to determine bit position


// extern char Operators[];          // Math operators 
char Operators[] = {'+','-','*','/','.','=',0};          // Math operators 

 
#ifdef KEYPAD_TEST
 
int            main(void)
{

    int             i;
    static int             ch;
    static byte            KeyValue;

    // Initialize variables here

  
    // Setup timers 

    SetupTimers();                                         
 
    // Initialize Keypad variables

    InitializeKeypad();
    
 	// Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
	InitializeUart1(); 

    PrintToScreen("\r Vex Keypad Controller Program \r");

#ifdef NCD_TEST

    printf("Start NCD Test ... \n\n");

    // Test the NCD function here

    KeyValue = NCD(128);
    PrintToScreen("NCD(128) = ");
    dec(KeyValue);
    printf("\r\n");

    KeyValue = NCD(64);
    PrintToScreen("NCD(64) = ");
    dec(KeyValue);
    printf("\r\n");


    KeyValue = NCD(32);
    PrintToScreen("NCD(32) = ");
    dec(KeyValue);
    printf("\r\n");


    KeyValue = NCD(16);
    PrintToScreen("NCD(16) = ");
    dec(KeyValue);
    printf("\r\n");

    KeyValue = NCD(8);
    PrintToScreen("NCD(8) = ");
    dec(KeyValue);
    printf("\r\n");


    KeyValue = NCD(4);
    PrintToScreen("NCD(4) = ");
    dec(KeyValue);
    printf("\r\n");


    KeyValue = NCD(2);
    PrintToScreen("NCD(2) = ");
    dec(KeyValue);
    printf("\r\n");


    KeyValue = NCD(1);
    PrintToScreen("NCD(1) = ");
    dec(KeyValue);
    printf("\r\n");


    KeyValue = NCD(10);
    PrintToScreen("NCD(10) = ");
    dec(KeyValue);
    printf("\r\n");

    KeyValue = NCD(5);
    PrintToScreen("NCD(5) = ");
    dec(KeyValue);
    printf("\r\n");

    KeyValue = NCD(0);
    PrintToScreen("NCD(0) = ");
    dec(KeyValue);
    printf("\r\n");

#endif

    // Demo loop which reads values from keypad and sends them to the host. It
    // Waits for press to indicate a keypress, then displays the key on the debug
    // screen.  Note that this code clears the press bit when done in order to
    // prepare for the next press.

    PrintToScreen("Vex Start KeyScan Test ... \r");
  
    while(1)
    {
    
       KeyValue = GetKey() - 48;					// Convert the key value from ASCII to binary
  
       // Send the scanned key to the host
       PrintToScreen("Key pressed = %d \r", KeyValue);
    
    }


}

#endif

//******************************************************************************
//* InitializeKeypad - Initialize the variables and Vex Controller's ports used by the 
//* keypad application.
//******************************************************************************

void            InitializeKeypad(void)
{

    // Initialize the ports used for keypad
    PORTF = 0x00;                                            // Clear PORTF - I06 .. I09
  	PORTH = 0x00;                                            // Clear PORTH - I013 .. I016

    // Initialize port I/O direction
   	TRISF = 0xFF;                                            //  Set PORT F to all outputs.
    TRISH = 0xFF;                                            //  Set PORT H to all inputs
  
  	// Configure PORT CN pull-up resistors enabled
	//ConfigCNPullups(0xFFF);					// Enable CN pin pullups (inputs) 
	//ConfigCNPullups(0x0000);					// Disable CN pin pullups (inputs)  

}
//******************************************************************************
//* KeyScan - Scan a 16-key matrix keypad for operator keystroke using the Vex 
//* Controller's I/O ports.
//******************************************************************************

int             KeyScan(void)
{
    int  i;                                     // Loop index
    int  j;                                     // Loop index
    int PORTHValue;                            // Value read from port B
   
    //pause(20); 											// Debounce keypad input
     
    do 
    {
     	// Wait for all keys up 
    	PORTF = 0x00;
    	TRISF = 0x00;
    	TRISH = 0xFF;
       //PrintToScreen("trace 1: PORTH  = %04X \r\n", (int) PORTH );
    
    } while ((PORTH >> 4) != 0xF);
   		
    // Wait a bit to allow the key to settle 
    Wait(20); 											
    
    // Debounce the keypad here
    for (i = 0; i < 4; i++)
    {

        // Discharge the keyboard
        PORTF = 0xFF;
        TRISF = 0x00;
        TRISH = 0x00;

        // Send PIO mode command to set port direction to input (This works for any row)
        TRISH = 0xFF;

        // Drive all columns low to determine if a key has been pressed (Port F pullups should be enabled).
        //Low(PORT_A, i);
        SetDigitalOutput(i+6, LOW);

        PORTF = Cols[i];
        Wait(1);

		// Save Port B value
        //PORTHValue = PORTH;
        //PORTHValue = (PORTH >> 4) & 0x0000F;
        PORTHValue = PORTH + (Cols[i] & 0x000F);
        //PORTHValue =  (PORTHValue << 4) + 
      
        //PrintToScreen("trace 2: PORTH = %04X, PORTHValue = %04X \r\n", (int) PORTH, (int) PORTHValue);
        //Wait(100);

        // Look-up the key scanned in table
        for (j = 0; j < 16; j++)
        {
            if (PORTHValue == KeyLookupTable[j])
            {
                key = j;
                goto ExitLoop;
            }
        }
        key = 0xFF;

    }

ExitLoop:

    if (key != 0xFF)
    {
       
   
        //PrintToScreen("trace 3:  key = %d \r\n", key);
 
        if (db == 1)
            goto done;                                     // Already responded to this press, so done.
        db = 1;                                            // Set debounce and keypress flags.
        press = 1;

    }
done:

    db = 0;                                                // Reset the debounce bit

    return key;                                            // Return keystroke to caller

}


//******************************************************************************
//* GetKey - Scans a character from the keypad or reads it from the serial 
//* port (USART).    
//******************************************************************************
char GetKey(void)
{
	char TheKey;

#ifdef KEYPAD_DRIVER

	// Wait for a valid key from the keypad
	      		
	do 
	{		  
		TheKey = KeyScan();
	} while (press == 0);
			  
	press = 0;
	
	if ((TheKey >=0) && (TheKey <= 9)) 
	{
		TheKey += 48;            // Convert the key to ASCII "0".."9"
	}
	else
	{
		TheKey = Operators[TheKey - 10];
	}
	//PrintToScreen("%d \r\n", TheKey);
#else
	TheKey = getch();
#endif
	
	return TheKey;
}


//******************************************************************************
//* NCD - Priority encoder operator, finds the position of the most significant bit.  
//******************************************************************************

int             NCD(byte Value)
{
    int             i;
    int             Length;
    int             temp3;
    Length = 0;

    for (i = 7; i >= 0; i--)
    {
        temp3 = Table[i] & Value;
        if (temp3 == Table[i])
        {
            //Length = i + 1;
            Length = i;
            break;
        }
    }

    return (Length);
}

  

