#include "BuiltIns.h"
#include "ifi_default.h"
#include "ifi_aliases.h"
#include "my_types.h"
#include "lcd.h"

#include "API.h"

/*  
Name: WPIKeypad.c
Description:
Vex Keypad Experiment 
In this experiment we connect a keypad to the Vex Controller and use it as a simple numeric data entry device or to 
input menu selections for an LCD based User Interface.  
 
We start out just transfering data a byte at a time via the 8-Bit Parallel Interface.

This function must be here for a competition project. It 
is automatically referenced by WPILib at startup and run.  
At that point the SetCompetitionMode function sets the  
competition mode. Basically, a mode of 0 is the default  
(without the IO_Initialization function) and runs a main  
function only.  SetCompetitionMode(1) runs a competition project as shown. 
#define PORT_1 1
#define PORT_2 2
#define CHANNEL_1 1
#define CHANNEL_2 2
#define CHANNEL_3 3
#define CHANNEL_4 4
#define CHANNEL_5 5
#define CHANNEL_6 6
#define CHANNEL_5_TOP 1
#define CHANNEL_5_BOTTOM 2
#define CHANNEL_6_TOP 3
#define CHANNEL_6_BOTTOM 4

// Used in User_Initialization routine in user_routines.c file. 
// Used to set pins as INPUTS (analog or digital) or OUTPUTS (digital only). 
#define INPUT   1
#define OUTPUT  0

/* 
// Used in User_Initialization routine in user_routines.c file. 
// Used to set pins as INPUTS (analog or digital) or OUTPUTS (digital only). 
#define IO1             TRISAbits.TRISA0
#define IO2             TRISAbits.TRISA1
#define IO3             TRISAbits.TRISA2
#define IO4             TRISAbits.TRISA3
#define IO5             TRISAbits.TRISA5
#define IO6             TRISFbits.TRISF0
#define IO7             TRISFbits.TRISF1
#define IO8             TRISFbits.TRISF2
#define IO9             TRISFbits.TRISF3
#define IO10            TRISFbits.TRISF4
#define IO11            TRISFbits.TRISF5
#define IO12            TRISFbits.TRISF6
#define IO13            TRISHbits.TRISH4
#define IO14            TRISHbits.TRISH5
#define IO15            TRISHbits.TRISH6
#define IO16            TRISHbits.TRISH7

// Aliases used to read the pins when used as INPUTS. 
#define rc_dig_in01     PORTAbits.RA0
#define rc_dig_in02     PORTAbits.RA1
#define rc_dig_in03     PORTAbits.RA2
#define rc_dig_in04     PORTAbits.RA3
#define rc_dig_in05     PORTAbits.RA5
#define rc_dig_in06     PORTFbits.RF0
#define rc_dig_in07     PORTFbits.RF1
#define rc_dig_in08     PORTFbits.RF2
#define rc_dig_in09     PORTFbits.RF3
#define rc_dig_in10     PORTFbits.RF4
#define rc_dig_in11     PORTFbits.RF5
#define rc_dig_in12     PORTFbits.RF6
#define rc_dig_in13     PORTHbits.RH4
#define rc_dig_in14     PORTHbits.RH5
#define rc_dig_in15     PORTHbits.RH6
#define rc_dig_in16     PORTHbits.RH7

// Aliases used to drive the pins when used as OUTPUTS. 
#define rc_dig_out01    LATAbits.LATA0
#define rc_dig_out02    LATAbits.LATA1
#define rc_dig_out03    LATAbits.LATA2
#define rc_dig_out04    LATAbits.LATA3
#define rc_dig_out05    LATAbits.LATA5
#define rc_dig_out06    LATFbits.LATF0
#define rc_dig_out07    LATFbits.LATF1
#define rc_dig_out08    LATFbits.LATF2
#define rc_dig_out09    LATFbits.LATF3
#define rc_dig_out10    LATFbits.LATF4
#define rc_dig_out11    LATFbits.LATF5
#define rc_dig_out12    LATFbits.LATF6
#define rc_dig_out13    LATHbits.LATH4
#define rc_dig_out14    LATHbits.LATH5
#define rc_dig_out15    LATHbits.LATH6
#define rc_dig_out16    LATHbits.LATH7

// *
// *-------------------------------------------------------------------------------
// *---------- Aliases for INTERRUPTS connectors (inputs) -------------------------
// *-------------------------------------------------------------------------------
// *  Below are aliases for the input pins labelled INTERRUPTS on the VEX Robot Controller.
// *  These pins are connected to PULL-UPS, meaning they are normally HIGH (1).
// 
#define rc_dig_int1     PORTBbits.RB2
#define rc_dig_int2     PORTBbits.RB3
#define rc_dig_int3     PORTBbits.RB4
#define rc_dig_int4     PORTBbits.RB5
#define rc_dig_int5     PORTBbits.RB6
#define rc_dig_int6     PORTBbits.RB7


*/ 

 

//*****************************************************************************************************
// Define keypad constants here...
#define LOW  0                                  // Logic low
#define HIGH 1                                  // Logic high

#define ENTER  14             // "#" is the Enter key assigned to the keypad


// LCD Buffer used to format and display text
char Buffer[80];

// Power of 10 lookup table 
const int PowerOfTen[3] = {100, 10, 1}; 

// Cycle counter used to update the LCD Display
int Cycle = 0;

/***************************************************************************************
 * IO_Initialization -  This procedure initializes the Vex Controller and I/O Ports. 
 ***************************************************************************************/
void IO_Initialization(void)
{ 
	SetCompetitionMode(0,1); // VEX Runs main
  //SetCompetitionMode(1, 1); // VEX
 
   //Controller Port   #: 01,02,03,04,05,06,07,08,09,10,11,12,13,14,15,16 
   //DefineControllerIO(4, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0); 
   // The first parameter is the number of analog input ports.
   //                        01     02     03     04     05     06     06     07     08     09      10      11     13      14      15      16
   //DefineControllerIO ( 5, INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
  // DefineControllerIO ( 2, INPUT ,INPUT ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
   //DefineControllerIO (   0, INPUT ,INPUT ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
   DefineControllerIO (   0, OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,INPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
 
} 
 

/***************************************************************************************
 *  Initialize - This function is run immediately when the robot is powered on 
 *  regardless of the field mode. 
 ***************************************************************************************/
void Initialize(void) 
{ 
	int i; 
	for (i = 0; i < 10; i++) 
	{ 
		PrintToScreen("Initialize %d\r", i); 
		//printf("Initialize %d\r", i); 
		Wait(500); 
	} 
} 

//******************************************************************************************
//* main -  The main program is used to generate a test ramp 0..255 and
//* send it to the controller.
//****************************************************************************************** 
void main(void)
{ 	
	unsigned char port;
	unsigned char function;
	unsigned char pwm;
	unsigned char invert;
    unsigned char LeftPWM;
    unsigned char RightPWM;
    unsigned int Joy_X_1;
    unsigned int Joy_Y_1;
    unsigned int Joy_X_2;
    unsigned int Joy_Y_2;
	int i;
    int Data_Byte = 0;
    int KeyValue = 0;
    int Direction = 0; // Default to Counter Clockwise (CCW) motor direction (CCW=0, CW=1)
    int Speed = 0;     // Motor speed (0..120 = CCW; 120..128 = Stop; 129..255 = CW) - These values may need to be calibrated!


	// Open serial ports here
	OpenSerialPortOne(BAUD_115200);
 
    // Set digital I/O port directions and initial values
    IO_Initialization();
 
    PrintToScreen("\r\n Initializing the Vex Keypad Interface... \r\n"); 
    // Configure ports and initialize variables used by the Keypad application
    InitializeKeypad();
 
    PrintToScreen("\r\n Enter keystroke using keypad... \r\n"); 
    // Initialize the LCD Display
	InitializeLCD();
    Wait(20);

   // VEX User Interface (UI)
   // Test 1 - A practical example - enter the speed and direction for a specific 
   // VEX Motor Display each kystroke entered from the keypad to the LCD. Use the 
   // WPILIB to run the VEX motors and servos.
 

    // Define robot with two wheels
  	TwoWheelDrive(1, 2);  	// This code works!!!

    // Clear the LCD and move the cursor to home position
    lcd_clear();


   //Format the Message to be displayed on the LCD 
    Buffer[0] = 'E';
    Buffer[1] = 'n';
    Buffer[2] = 't';
    Buffer[3] = 'e';
    Buffer[4] = 'r';
    Buffer[5] = ' ';
    Buffer[6] = 'S';
    Buffer[7] = 'p';
    Buffer[8] = 'e';
    Buffer[9] = 'e';
    Buffer[10] = 'd';
    Buffer[11] = '(';
    Buffer[12] = '0';
    Buffer[13] = '.';
    Buffer[14] = '.';
    Buffer[15] = '2';
    Buffer[16] = '5';
    Buffer[17] = '5';
    Buffer[18] = ')';
    Buffer[19] = ':';      
    Buffer[20] = 0;     // String terminator (null character)  

   
    // Position the cursor to the first line
    lcd_goto(0,2);

    //Wait(1);

    // Send message to the LCD Display
    lcd_puts((char *) Buffer);  

   // Send message to the Serial Terminal if available
  	printf("Enter Speed (0..255) \r\n", KeyValue);
    //Format the Message to be displayed on the LCD 
    Buffer[0] = 'S ';
    Buffer[1] = 'p';
    Buffer[2] = 'e';
    Buffer[3] = 'e';
    Buffer[4] = 'd';
    Buffer[5] = ' ';
    Buffer[6] = '=';
    Buffer[7] = ' ';
    Buffer[8] = 0;     // String terminator (null character)
 
  

    Speed = 0;
    i = 0;
    while(1)
    {
 
     	KeyValue = GetKey();				// Convert the key value from ASCII to binary

        // Check the range of the Key Value to make sure is a digit between 0 and 9
        if ((KeyValue >=0) && (KeyValue <=9))
        {
           // Enter the Speed one digit at a time
     
           if (i<3)  
           {
              Speed += (int)KeyValue*PowerOfTen[i];
              i++;

              // Position the cursor
              lcd_goto(1,2);
              //Wait(1);

              // Display formatted text to the LCD
    	      lcd_puts((char *) Buffer);  
              lcd_printdec(Speed);
 
    	      // Send the scanned key to the host serial terminal also
       	      // printf("Key = %d \r\n", KeyValue);
 
   	          // Send the scanned key to the host serial terminal also
       	      //printf("Speed = %d \r\n", Speed);
              
           }
           else
           {
              // Set the VEX Motor # 1 Speed 
	          SetPWM (1,Speed);
           
              // Reset digit count and Speed
              i = 0;
              Speed = 0;
              
           }
        }
        else if (KeyValue == ENTER)
        {
           // Process the speed value entered 
   	       // Send the scanned key to the host serial terminal also
       	   printf("Current Motor Speed = %d \r\n", Speed);
 
           // Set the VEX Motor # 1 Speed 
	       SetPWM (1,Speed);
 
           // Reset digit count and Speed
            i = 0;
            Speed = 0;
        }
    }


    // Test 2 - Display each keystroke entered from the keypad to the LCD.  Works!!!
    // Clear the LCD and move the cursor to home position
    lcd_clear();
    Wait(2);

     //Format the Message to be displayed on the LCD 
    Buffer[0] = ' ';
    Buffer[1] = 'E';
    Buffer[2] = 'n';
    Buffer[3] = 't';
    Buffer[4] = 'e';
    Buffer[5] = 'r';
    Buffer[6] = ' ';
    Buffer[7] = 'K';
    Buffer[8] = 'e';
    Buffer[9] = 'y';
    Buffer[10] = ' ';
    Buffer[11] = '(';
    Buffer[12] = '0';
    Buffer[13] = '.';
    Buffer[14] = '.';
    Buffer[15] = '9';
    Buffer[16] = ')';
    Buffer[17] = ':';      
    Buffer[18] = 0;     // String terminator (null character)
   
    // Position the cursor to the first line
    lcd_goto(0,2);

    Wait(1);

    // Send message to the LCD Display
    lcd_puts((char *) Buffer);  

   // Send message to the Serial Terminal if available
  	printf("Enter Key ('0'..'9') \r\n", KeyValue);
 
  //Format the Message to be displayed on the LCD 
    Buffer[0] = ' ';
    Buffer[1] = 'K';
    Buffer[2] = 'e';
    Buffer[3] = 'y';
    Buffer[4] = ' ';
    Buffer[5] = 'p';
    Buffer[6] = 'r';
    Buffer[7] = 'e';
    Buffer[8] = 's';
    Buffer[9] = 's';
    Buffer[10] = 'e';
    Buffer[11] = 'd';
    Buffer[12] = ' ';
    Buffer[13] = '=';
    Buffer[14] = ' ';
    Buffer[15] = 0;     // String terminator (null character)


  
    // Demo loop which reads values from keypad and sends them to the LCD. It
    // Waits for press to indicate a keypress, then displays the key on the LCD
    // Display.  Note that this code clears the press bit when done in order to
    // prepare for the next press.
    while(1)
    {
 
    	//KeyValue = GetKey() - 48;			// Convert the key value from ASCII to binary
    	KeyValue = GetKey();				// Convert the key value from ASCII to binary

        // Check the range of the Key Value to make sure is a digit between 0 and 9
        if ((KeyValue >=0) && (KeyValue <=9))
        {
           // Position the cursor
           lcd_goto(1,2);
           Wait(1);

           // Display formatted text to the LCD
    	   lcd_puts((char *) Buffer);  
           lcd_printdec(KeyValue);
           //Wait(2000);
 
    	   // Send the scanned key to the host serial terminal also
       	   //printf("Key pressed = %d \r\n", KeyValue);
        }
   	 
    }


 

    // Test 2 - Display each keystroke entered from the keypad to the LCD
    // Clear the LCD and move the cursor to home position
    lcd_clear();
  
     //Format the Message to be displayed on the LCD 
    Buffer[0] = ' ';
    Buffer[1] = 'E';
    Buffer[2] = 'n';
    Buffer[3] = 't';
    Buffer[4] = 'e';
    Buffer[5] = 'r';
    Buffer[6] = ' ';
    Buffer[7] = 'K';
    Buffer[8] = 'e';
    Buffer[9] = 'y';
    Buffer[10] = ' ';
    Buffer[11] = '(';
    Buffer[12] = '0';
    Buffer[13] = '.';
    Buffer[14] = '.';
    Buffer[15] = '9';
    Buffer[16] = ')';
    Buffer[17] = ':';      
    Buffer[18] = 0;     // String terminator (null character)
   
    // Position the cursor to the first line
    lcd_goto(0,2);

    Wait(1);

    // Send message to the LCD Display
    lcd_puts((char *) Buffer);  

   // Send message to the Serial Terminal if available
  	printf("Enter Key ('0'..'9') \r\n", KeyValue);
 
  //Format the Message to be displayed on the LCD 
    Buffer[0] = ' ';
    Buffer[1] = 'K';
    Buffer[2] = 'e';
    Buffer[3] = 'y';
    Buffer[4] = ' ';
    Buffer[5] = 'p';
    Buffer[6] = 'r';
    Buffer[7] = 'e';
    Buffer[8] = 's';
    Buffer[9] = 's';
    Buffer[10] = 'e';
    Buffer[11] = 'd';
    Buffer[12] = ' ';
    Buffer[13] = '=';
    Buffer[14] = ' ';
    Buffer[15] = 0;     // String terminator (null character)
    while(1)
    {
 
    	//KeyValue = GetKey() - 48;			// Convert the key value from ASCII to binary
    	KeyValue = GetKey();				// Convert the key value from ASCII to binary

        // Check the range of the Key Value to make sure is a digit between 0 and 9
        if ((KeyValue >=0) && (KeyValue <=9))
        {
           // Position the cursor
           lcd_goto(1,2);
           Wait(1);

           // Display formatted text to the LCD
    	   lcd_puts((char *) Buffer);  
           lcd_printdec(KeyValue);
 
    	   // Send the scanned key to the host serial terminal also
       	   printf("Key pressed = %d \r\n", KeyValue);
        }
   	 
    }

  	 while (1)   // Forever
  	{
 		for (i=0; i<255; i++)
		{
			printf("Testing %d times!!!\n", i);
			Wait(500);
			
		}
    }
 
    // Configure ports and initialize variables used by the Keypad application
    InitializeKeypad();

    // Demo loop which reads values from keypad and sends them to the host. It
    // Waits for press to indicate a keypress, then displays the key on the debug
    // screen.  Note that this code clears the press bit when done in order to
    // prepare for the next press.

   
    while(1)
    {
    
    	//KeyValue = GetKey() - 48;			// Convert the key value from ASCII to binary
    	KeyValue = GetKey();				// Convert the key value from ASCII to binary
  
    	// Send the scanned key to the host

    	PrintToScreen("Key pressed = %d \r\n", KeyValue);
   	 

    }
/*
  strcpypgm2ram (ifi_printfBufr,(rom char *) bufr); // Move from flash to ram
  len = (int) strlen((const char *)ifi_printfBufr);
  if (len > 80) len = 80;
*/

/*
    for (i=0; i<255; i++)
    {

		// Move the selected motors.  Works!!!
		SetPWM (1,i);
		SetPWM (2,i);		
        SetPWM (3,i);
		SetPWM (4,i);
        Wait(500);
    }
*/
 
 
   
}
