#include "BuiltIns.h"
#include "ifi_default.h"
#include "ifi_aliases.h"
#include "my_types.h"

//#include "API.h"

/*  
Name: WPIParallel.c
Description:
8-Bit Parallel Interface Experiment
This application will allow communications between two Vex Controllers.  The first Vex Controller
is the Master Controller and the second Vex Conroller is the co-processor. Spliting up the pocessing
in this manner allows the first Vex Controller to run the algorithms and the robot behaviors while the
second Vex Controller just sends PWM commands to the motors or reads sensor data and Joystick positions 
send from the Vex RC Remote.  The WPILib functions are used to control the Vex motors and read the
Operator Input (OI) interface from the Vex RC Remote Control.

We start out just transfering data a byte at a time via the 8-Bit Parallel Interface.

This function must be here for a competition project. It 
is automatically referenced by WPILib at startup and run.  
At that point the SetCompetitionMode function sets the  
competition mode. Basically, a mode of 0 is the default  
(without the IO_Initialization function) and runs a main  
function only.  SetCompetitionMode(1) runs a competition project as shown. 
#define PORT_1 1
#define PORT_2 2
#define CHANNEL_1 1
#define CHANNEL_2 2
#define CHANNEL_3 3
#define CHANNEL_4 4
#define CHANNEL_5 5
#define CHANNEL_6 6
#define CHANNEL_5_TOP 1
#define CHANNEL_5_BOTTOM 2
#define CHANNEL_6_TOP 3
#define CHANNEL_6_BOTTOM 4

// Used in User_Initialization routine in user_routines.c file. 
// Used to set pins as INPUTS (analog or digital) or OUTPUTS (digital only). 
#define INPUT   1
#define OUTPUT  0

/* 
// Used in User_Initialization routine in user_routines.c file. 
// Used to set pins as INPUTS (analog or digital) or OUTPUTS (digital only). 
#define IO1             TRISAbits.TRISA0
#define IO2             TRISAbits.TRISA1
#define IO3             TRISAbits.TRISA2
#define IO4             TRISAbits.TRISA3
#define IO5             TRISAbits.TRISA5
#define IO6             TRISFbits.TRISF0
#define IO7             TRISFbits.TRISF1
#define IO8             TRISFbits.TRISF2
#define IO9             TRISFbits.TRISF3
#define IO10            TRISFbits.TRISF4
#define IO11            TRISFbits.TRISF5
#define IO12            TRISFbits.TRISF6
#define IO13            TRISHbits.TRISH4
#define IO14            TRISHbits.TRISH5
#define IO15            TRISHbits.TRISH6
#define IO16            TRISHbits.TRISH7

// Aliases used to read the pins when used as INPUTS. 
#define rc_dig_in01     PORTAbits.RA0
#define rc_dig_in02     PORTAbits.RA1
#define rc_dig_in03     PORTAbits.RA2
#define rc_dig_in04     PORTAbits.RA3
#define rc_dig_in05     PORTAbits.RA5
#define rc_dig_in06     PORTFbits.RF0
#define rc_dig_in07     PORTFbits.RF1
#define rc_dig_in08     PORTFbits.RF2
#define rc_dig_in09     PORTFbits.RF3
#define rc_dig_in10     PORTFbits.RF4
#define rc_dig_in11     PORTFbits.RF5
#define rc_dig_in12     PORTFbits.RF6
#define rc_dig_in13     PORTHbits.RH4
#define rc_dig_in14     PORTHbits.RH5
#define rc_dig_in15     PORTHbits.RH6
#define rc_dig_in16     PORTHbits.RH7

// Aliases used to drive the pins when used as OUTPUTS. 
#define rc_dig_out01    LATAbits.LATA0
#define rc_dig_out02    LATAbits.LATA1
#define rc_dig_out03    LATAbits.LATA2
#define rc_dig_out04    LATAbits.LATA3
#define rc_dig_out05    LATAbits.LATA5
#define rc_dig_out06    LATFbits.LATF0
#define rc_dig_out07    LATFbits.LATF1
#define rc_dig_out08    LATFbits.LATF2
#define rc_dig_out09    LATFbits.LATF3
#define rc_dig_out10    LATFbits.LATF4
#define rc_dig_out11    LATFbits.LATF5
#define rc_dig_out12    LATFbits.LATF6
#define rc_dig_out13    LATHbits.LATH4
#define rc_dig_out14    LATHbits.LATH5
#define rc_dig_out15    LATHbits.LATH6
#define rc_dig_out16    LATHbits.LATH7

// *
// *-------------------------------------------------------------------------------
// *---------- Aliases for INTERRUPTS connectors (inputs) -------------------------
// *-------------------------------------------------------------------------------
// *  Below are aliases for the input pins labelled INTERRUPTS on the VEX Robot Controller.
// *  These pins are connected to PULL-UPS, meaning they are normally HIGH (1).
// 
#define rc_dig_int1     PORTBbits.RB2
#define rc_dig_int2     PORTBbits.RB3
#define rc_dig_int3     PORTBbits.RB4
#define rc_dig_int4     PORTBbits.RB5
#define rc_dig_int5     PORTBbits.RB6
#define rc_dig_int6     PORTBbits.RB7


*/ 

/*
// Hardware interface with the  Vex Controller (using Microchip PIC18F8520 Pin numbering)
// 8-Bit Parallel Port Control signals
#define CS_BAR_TRIS             TRISFbits.TRISF9
#define WR_BAR_TRIS             TRISFbits.TRISF10
#define RD_BAR_TRIS             TRISFbits.TRISF11
#define BUSY_BAR_TRIS          TRISFbits.TRISF5

#define CS_BAR        rc_dig_out09      // Bits are shifted out this pin # to the MAX7219.
#define WR_BAR        rc_dig_out10      // Data valid on rising edge of this clock (pin RD5).
#define RD_BAR        rc_dig_out11      // This control pin tells the 7219 to transfer data to LEDs.  
#define BUSY_BAR          		PORTFbits.RF5



#define RD_BAR             		LATFbits.LATF3
#define WR_BAR             		LATFbits.LATF4
#define BUSY_BAR          		PORTFbits.RF5
*/

//*****************************************************************************************************
// Define stepper motor constants here...
#define LOW  0                                  // Logic low
#define HIGH 1                                  // Logic high

// Hardware interface with the  Vex Controller (using Vex Controller Pin numbering)
 
#define CS_BAR           9      			// Chip Select to enable the peripheral controller parallel interface
#define WR_BAR        	10      			// Write to the peripheral controller
#define RD_BAR       	11      			// Read from the peripheral controller
#define BUSY_BAR       12      			// Signal indicating that the peripheral is ready to accept another parallel command

//*****************************************************************************************************
 
// Define data structure for virtual 8-Bit Port.  This can be 
// extended to 16-Bit or 32-Bit virtual ports similar to the BS2/BSX Stamp
enum Port_T {PORT_A, PORT_B, PORT_C, PORT_D, PORT_E, PORT_F, PORT_G, PORT_H, PORT_J}; 

typedef volatile union  {
    byte Port_Value;		// The port value as a byte
    struct {
        byte VR_BIT_7:1;
        byte VR_BIT_6:1;
        byte VR_BIT_5:1;
        byte VR_BIT_4:1;
        byte VR_BIT_3:1;
        byte VR_BIT_2:1;
        byte VR_BIT_1:1;
        byte VR_BIT_0:1;
/*
        byte VR_BIT_0:1;
        byte VR_BIT_1:1;
        byte VR_BIT_2:1;
        byte VR_BIT_3:1;
        byte VR_BIT_4:1;
        byte VR_BIT_5:1;
        byte VR_BIT_6:1;
        byte VR_BIT_7:1;  
*/   
    } ;
 
} VR_8_BIT_PORT_T;   

// Declare local function headers here
VR_8_BIT_PORT_T Read_Virtual_Port(byte Port_ID);

VR_8_BIT_PORT_T VR_Port_Data;	// Virtual port data (8-Bits)

void Write_Virtual_Port(byte Port_ID, VR_8_BIT_PORT_T The_Data);
byte Read_Byte(void);
 
/***************************************************************************************
 * IO_Initialization -  This procedure initializes the Vex Controller and I/O Ports. 
 ***************************************************************************************/
void IO_Initialization(void)
{ 
	SetCompetitionMode(0,1); // VEX Runs main
  //SetCompetitionMode(1, 1); // VEX
 
   //Controller Port   #: 01,02,03,04,05,06,07,08,09,10,11,12,13,14,15,16 
   //DefineControllerIO(4, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0); 
   // The first parameter is the number of analog input ports.
   //                        01     02     03     04     05     06     06     07     08     09      10      11     13      14      15      16
   //DefineControllerIO ( 5, INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
  // DefineControllerIO ( 2, INPUT ,INPUT ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
   //DefineControllerIO (   0, INPUT ,INPUT ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
   DefineControllerIO (   0, OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,INPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
 
} 
 

/***************************************************************************************
 *  Initialize - This function is run immediately when the robot is powered on 
 *  regardless of the field mode. 
 ***************************************************************************************/
void Initialize(void) 
{ 
	int i; 
	for (i = 0; i < 10; i++) 
	{ 
		PrintToScreen("Initialize %d\r", i); 
		//printf("Initialize %d\r", i); 
		Wait(500); 
	} 
} 


 
//******************************************************************************
//* Read_Virtual_Port - Read a byte of data from the selected Port,  
//* Port_ID range (PORT_A..PORTE).  It reads the bits from the
//* assigned ports and saves them to Virtual Port A, bits A(0:5), Port F bits(0:2)  
//*   
//******************************************************************************

VR_8_BIT_PORT_T Read_Virtual_Port(byte Port_ID)
{
	VR_8_BIT_PORT_T The_Virtual_Port;

	switch(Port_ID)
	{
		case PORT_A:
		{
/*
			// Set PORT A bits to inputs
			TRISAbits.TRISA0 = 1;
			TRISAbits.TRISA1 = 1;
			TRISAbits.TRISA2 = 1;
			TRISAbits.TRISA3 = 1;
			TRISAbits.TRISA5 = 1;

			// Set PORT F bits to inputs
			TRISFbits.TRISF0 = 1;
			TRISFbits.TRISF1 = 1;
			TRISFbits.TRISF2 = 1;
*/
            //DefineControllerIO (   0, INPUT ,INPUT ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
            IO1 = IO2 = IO3 = IO4 = IO5 = IO6 = IO7 = IO8 = INPUT;

			// Copy the virtual port bits to the physical port bit assignments using  MSB first (Big Endian?)
            The_Virtual_Port.VR_BIT_7 = PORTAbits.RA0;
		 	The_Virtual_Port.VR_BIT_6 = PORTAbits.RA1;;
            The_Virtual_Port.VR_BIT_5 = PORTAbits.RA2;
		 	The_Virtual_Port.VR_BIT_4 = PORTAbits.RA3;
            The_Virtual_Port.VR_BIT_3 = PORTAbits.RA5;;
		 	The_Virtual_Port.VR_BIT_2 = PORTFbits.RF0;
		 	The_Virtual_Port.VR_BIT_1 = PORTFbits.RF1;
		 	The_Virtual_Port.VR_BIT_0 = PORTFbits.RF2;

#ifdef NEW_CODE
            
			// Copy the virtual port bits to the physical port bit assignments using LSB first (Little Endian?)
            The_Virtual_Port.VR_BIT_0 = PORTAbits.RA0;
		 	The_Virtual_Port.VR_BIT_1 = PORTAbits.RA1;;
            The_Virtual_Port.VR_BIT_2 = PORTAbits.RA2;
		 	The_Virtual_Port.VR_BIT_3 = PORTAbits.RA3;
            The_Virtual_Port.VR_BIT_4 = PORTAbits.RA5;;
		 	The_Virtual_Port.VR_BIT_5 = PORTFbits.RF0;
		 	The_Virtual_Port.VR_BIT_6 = PORTFbits.RF1;
		 	The_Virtual_Port.VR_BIT_7 = PORTFbits.RF2;
#endif
			break;
		}

		case PORT_D:
		{
			The_Virtual_Port.Port_Value = PORTD;
			break;
		}

	}
	return The_Virtual_Port;	
}


//******************************************************************************
//* Write_Virtual_Port - Write a byte of data from the selected port, 
//* Port_ID range (PORT_A..PORTE). It Distributes the bits from the virtual data byte to the
//* Port_ID range (PORT_A..PORTE) using the following
//* assigned ports and saves them to Virtual Port A, bits A(0:5), Port F bits(0:2)  
//******************************************************************************
void Write_Virtual_Port(byte Port_ID, VR_8_BIT_PORT_T The_Data)
{
	// Distribute the bits from the virtual data byte to the
	// assigned for Virtual Port A, bits to Virtual Port A, bits A(0:5), Port F bits(0:2) 
	switch(Port_ID)
	{
		case PORT_A:
		{
/*
			// Set PORT A bits to outputs
			TRISAbits.TRISA0 = 0;
			TRISAbits.TRISA1 = 0;
			TRISAbits.TRISA2 = 0;
			TRISAbits.TRISA3 = 0;
			TRISAbits.TRISA5 = 0;

			// Set PORT F bits to outputs
			TRISFbits.TRISF0 = 0;
			TRISFbits.TRISF1 = 0;
			TRISFbits.TRISF2 = 0;
*/
            //DefineControllerIO (   0, OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
            IO1 = IO2 = IO3 = IO4 = IO5 = IO6 = IO7 = IO8 = OUTPUT;

			// Copy the virtual port bits to the physical port bit assignments using MSB first (Big Endian?)
            LATAbits.LATA0 = The_Data.VR_BIT_7;
			LATAbits.LATA1 = The_Data.VR_BIT_6;
			LATAbits.LATA2 = The_Data.VR_BIT_5;
			LATAbits.LATA3 = The_Data.VR_BIT_4;
			LATAbits.LATA5 = The_Data.VR_BIT_3;
			LATFbits.LATF0 = The_Data.VR_BIT_2;
			LATFbits.LATF1 = The_Data.VR_BIT_1;
			LATFbits.LATF2 = The_Data.VR_BIT_0;

#ifdef NEW_CODE
            
			// Copy the virtual port bits to the physical port bit assignments using LSB first (Little Endian?)
            LATAbits.LATA0 = The_Data.VR_BIT_0;
			LATAbits.LATA1 = The_Data.VR_BIT_1;
			LATAbits.LATA2 = The_Data.VR_BIT_2;
			LATAbits.LATA3 = The_Data.VR_BIT_3;
			LATAbits.LATA5 = The_Data.VR_BIT_4;
			LATFbits.LATF0 = The_Data.VR_BIT_5;
			LATFbits.LATF1 = The_Data.VR_BIT_6;
			LATFbits.LATF2 = The_Data.VR_BIT_7;
#endif
			break;
		}
 
		case PORT_D:
		{
			// Set PORT D bits to outputs
			TRISD = 0;

			PORTD = The_Data.Port_Value;

			break;
		}

	}
}


//******************************************************************************************
//* Read_Byte -  This routine reads a byte of data from the 8-Bit parallel interface into the 
//* Vex Controller.
//******************************************************************************************
byte Read_Byte(void)
{
	byte Data_Byte = 0;
	//PrintToScreen("Entering Read_Byte...\r");
    //BUSY_BAR_TRIS = 1;    	// Set BUSY_BAR pin direction to input         
    //RD_BAR_TRIS = 0;        // Set RD_BAR pin direction to output       
   
  	//PrintToScreen("Trace 1\r");

    // Check if co-processor is busy
    while (rc_dig_in12);  

	// Copy the Data byte to the 8-Bit parallel port

	// Set signal RD_BAR to LOW indicating to the PIC18F4550 USB Controller that it can put a byte of data
	// on the parallel bus so that it can be read in by the Vex Controller.   
	//RD_BAR = 0;
    SetDigitalOutput(RD_BAR, LOW);
   // Wait a bit
   // pause(1);

	// Set signal RD_BAR to HIGH indicating to the PIC18F4550 USB Controller that it can put a byte of data
	// on the parallel bus so that it can be read in by the Vex Controller.   
	//RD_BAR = 1;
 

	SetDigitalOutput(RD_BAR, HIGH);

   // Wait a bit
   // pause(1);

	//PrintToScreen("Trace 2\r");

 
	// Read the data from the 8-Bit Parallel Port
	VR_Port_Data = Read_Virtual_Port(PORT_A);
	//PrintToScreen("Leaving Read_Byte.\r");
   
	// Return the data byte
	return VR_Port_Data.Port_Value;
}


//******************************************************************************************
//* Write_Byte -  This routine writes a byte of data to the 8-Bit parallel interface so that
//* it can be read and processed by the PIC18F4550 USB Controller
//******************************************************************************************
 void Write_Byte(byte Data)
{ 
	//PrintToScreen("Entering Write_Byte...\r");
   
    //BUSY_BAR_TRIS = 1;    	// Set BUSY_BAR pin direction to input         
    //WR_BAR_TRIS = 0;        // Set RD_BAR pin direction to output       

  	//PrintToScreen("Trace 1\r");
	
	//PrintToScreen("Data_Byte read = %d \r", BUSY_BAR);
	//PrintToScreen("Data_Byte read = %d \r", PORTFbits.RF5);

 	//PrintToScreen("Trace 2\r");

	// Copy the Data byte to the 8-Bit parallel port
 	VR_Port_Data.Port_Value = Data;
	Write_Virtual_Port(PORT_A, VR_Port_Data);
 
    // Check if co-processor is busy
    while (rc_dig_in12);  

 	//PrintToScreen("Trace 3\r");
  
 	// Signal WR_BAR to LOW indicate to the PIC18F4550 USB Controller that there is valid data
	// on the parallel bus that needs to be read and processed.  
	//WR_BAR = 0;
    SetDigitalOutput(WR_BAR, LOW);

    // Wait a bit
   // pause(1);

 	// Signal WR_BAR to HIGH indicate to the PIC18F4550 USB Controller that there is valid data
	// on the parallel bus that needs to be read and processed.  
	//WR_BAR = 0;

    SetDigitalOutput(WR_BAR, HIGH);

    // Wait a bit
    //pause(1);
	//PrintToScreen("Leaving Write_Byte\r");

}

//******************************************************************************************
//* main -  The main program is used to generate a test ramp 0..255 and
//* send it to the controller.
//****************************************************************************************** 
void main(void)
{ 	
	unsigned char port;
	unsigned char function;
	unsigned char pwm;
	unsigned char invert;
    unsigned char LeftPWM;
    unsigned char RightPWM;
    unsigned int Joy_X_1;
    unsigned int Joy_Y_1;
    unsigned int Joy_X_2;
    unsigned int Joy_Y_2;
	int i;
    int Data_Byte = 0;

	// Open serial ports here
	//OpenSerialPortOne(BAUD_9600);
	//OpenSerialPortTwo(BAUD_9600);
	OpenSerialPortOne(BAUD_115200);
	OpenSerialPortTwo(BAUD_115200);

    // Set digital I/O port directions and initial values
    IO_Initialization();


    //IO9 = 0;    // RD_BAR_TRIS = 0;        // Set RD_BAR pin direction to output       
    //IO10 = 0;   // WR_BAR_TRIS = 0;     	// Set RD_BAR pin direction to output              

    //IO11 = 1;   // BUSY_BAR_TRIS = 1;    	// Set RD_BAR pin direction to input         

    //BUSY_BAR_TRIS = 1;    	// Set BUSY_BAR pin direction to input         
    //RD_BAR_TRIS = 0;        // Set RD_BAR pin direction to output       
    //WR_BAR_TRIS = 0;        // Set WR_BAR pin direction to output       
    //CS_BAR_TRIS = 0;        // Set CS_BAR pin direction to output       

    // Set Vex Controller I/O Directions
	// Set the Vex Controller Digital I/O to Outputs assignments here...
	SetDirection(RD_BAR, OUTPUT);             			// Direction for left stepper motor
	SetDirection(WR_BAR, OUTPUT);     						// Used to activate left stepper motor (pulse)
	SetDirection(CS_BAR, OUTPUT);       					// Selects half step sequence (high)

 
	SetDirection(BUSY_BAR, INPUT);     					// Selects one phase wave drive (high) or two phase (low)

     // Start with high levels 
	SetDigitalOutput(RD_BAR, HIGH);
	SetDigitalOutput(WR_BAR, HIGH);
	SetDigitalOutput(CS_BAR, HIGH);

    // Setup timers, configure ports and initialize variables
   //Test parallel port control signals.  Works!!!
    //SetupTimers();  
    PrintToScreen("\r\n Initializing the Vex Parallel Interface... \r\n"); 
    
    for (i=0; i<100; i++)
    {
       PrintToScreen("This is a test %d times \r\n", i); 
       //Wait(100};
    }
 
    Wait(1000);

  
     //******************************************************************************************  
     //* Test  writing data to Parallel Port. Works!!!
     //******************************************************************************************  
     while(1)
     {
        for (i=0; i<255; i++)
        {
           // Write a byte of data to the co-processor
           Write_Byte(i);
           //Write_Byte(0xAA);
 
        }
      }

  
    //******************************************************************************************  
    // Test reading from the 8-Bit parallel port here.  Works!!!
    //******************************************************************************************  
    while(1)
    {
       for (i=0; i<255; i++)
       {
         // Write a byte of data to the co-processor
         Data_Byte = Read_Byte();
         PrintToScreen("Data_Byte = %d \r\n", Data_Byte); 
         //Wait(500);
       }
     }


 //******************************************************************************************  
 // Test  writing data to Parallel Port. Works!!!
 //******************************************************************************************  
   // Set WR_BAR High for read
   SetDigitalOutput(WR_BAR, HIGH);
  //CS_BAR = HIGH;

  while(1)
  {
     for (i=0; i<4; i++)
     {
        LATA = i;
 
        // Check if co-processor is busy
        while (rc_dig_in12);  

        // Write 8-Bits to the Parallel Port   
          // Set WR_BAR Low for read
    	SetDigitalOutput(WR_BAR, LOW);

        pause(1);
        // Write 8-Bits to the Parallel Port   
       	SetDigitalOutput(WR_BAR, HIGH);
        pause(1);
 
     }
  }




     while(1)
     {
      PrintToScreen("Testing Write = HIGH \r\n"); 
 
	    SetDigitalOutput(1, HIGH);
	    SetDigitalOutput(2, HIGH);
	    SetDigitalOutput(3, HIGH);
	    SetDigitalOutput(4, HIGH);
	    SetDigitalOutput(5, HIGH); 
	    SetDigitalOutput(6, HIGH);
	    SetDigitalOutput(7, HIGH);
	    SetDigitalOutput(8, HIGH);

        PrintToScreen("Testing WR_BAR = HIGH \r\n"); 
        // Set WR_BAR High for read
    	SetDigitalOutput(WR_BAR, HIGH);

        PrintToScreen("Testing WR_BAR = LOW \r\n"); 
        // Set WR_BAR Low for read
    	SetDigitalOutput(WR_BAR, LOW);

        //PORTA = 0xFF;
        Wait(500);

        PrintToScreen("Testing Write = LOW \r\n"); 
	    SetDigitalOutput(1, LOW);
	    SetDigitalOutput(2, LOW);
	    SetDigitalOutput(3, LOW);
	    SetDigitalOutput(4, LOW);
	    SetDigitalOutput(5, LOW);
	    SetDigitalOutput(6, LOW);
	    SetDigitalOutput(7, LOW);
	    SetDigitalOutput(8, LOW);


        //PORTA = 0x00;

          PrintToScreen("Testing WR_BAR = HIGH \r\n"); 
        // Set WR_BAR High for read
    	SetDigitalOutput(WR_BAR, HIGH);

        PrintToScreen("Testing WR_BAR = LOW \r\n"); 
        // Set WR_BAR Low for read
    	SetDigitalOutput(WR_BAR, LOW);

        Wait(500);

     }
  //******************************************************************************************  
   // Test reading from the 8-Bit parallel port here
 //******************************************************************************************  
     // Set Parallel Port direction to input
    IO1 = IO2 = IO3 = IO4 = IO5 = IO6 = IO7 =IO8 = INPUT;      /* Used for limit switch inputs. */

    while(1)
    {
       for (i=0; i<255; i++)
       {
         // Write a byte of data to the co-processor
         Data_Byte = Read_Byte();
         PrintToScreen("Data_Byte = %d \r\n", Data_Byte); 
         Wait(500);
       }
     }


     // Test the parallel port write.  

     // Set WR_BAR High for read
     // Set data direction to output
  	 SetDirection(1, OUTPUT);       					// Selects half step sequence (high)
 	 SetDirection(2, OUTPUT);       					// Selects half step sequence (high)
 	 SetDirection(3, OUTPUT);       					// Selects half step sequence (high)
	 SetDirection(4, OUTPUT);       					// Selects half step sequence (high)
	 SetDirection(5, OUTPUT);       					// Selects half step sequence (high)
	 SetDirection(6, OUTPUT);       					// Selects half step sequence (high)
	 SetDirection(7, OUTPUT);       					// Selects half step sequence (high)
	 SetDirection(8, OUTPUT);       					// Selects half step sequence (high)



#ifdef NEW_CODE
    // Test the parallel port Read.  Works!!!

     // Set RD_BAR High for read
     SetDigitalOutput(RD_BAR, HIGH);
     PrintToScreen("Testing RD_BAR = HIGH \r\n"); 
     Wait(1000);
     while(1)
     {
        // Set RD_BAR Low for read
    	SetDigitalOutput(RD_BAR, LOW);
        PrintToScreen("Testing RD_BAR = LOW \r\n"); 

        Wait(1000);
 
        // Set RD_BAR High for read
    	SetDigitalOutput(RD_BAR, HIGH);
        PrintToScreen("Testing RD_BAR = HIGH \r\n"); 
        Wait(1000);

     }

    // Test the parallel port Read.  Works!!!

     // Set RD_BAR High for read
     SetDigitalOutput(RD_BAR, HIGH);
     PrintToScreen("Testing RD_BAR = HIGH \r\n"); 
     Wait(1000);
     while(1)
     {
        // Set RD_BAR Low for read
    	SetDigitalOutput(RD_BAR, LOW);
        PrintToScreen("Testing RD_BAR = LOW \r\n"); 

        Wait(1000);
 
        // Set RD_BAR High for read
    	SetDigitalOutput(RD_BAR, HIGH);
        PrintToScreen("Testing RD_BAR = HIGH \r\n"); 
        Wait(1000);

     }

   // Test the parallel port Chip Select. Works!!!

     // Set CS_BAR High for selecting the chip
     SetDigitalOutput(CS_BAR, HIGH);
     PrintToScreen("Testing CS_BAR = HIGH \r\n"); 
     Wait(1000);
     while(1)
     {
        // Set CS_BAR Low to select the chip
    	SetDigitalOutput(CS_BAR, LOW);
        PrintToScreen("Testing CS_BAR = LOW \r\n"); 

        Wait(1000);
 
        // Set CS_BAR High for read
    	SetDigitalOutput(CS_BAR, HIGH);
        PrintToScreen("Testing CS_BAR = HIGH \r\n"); 
        Wait(1000);

     }

     //TRISA = 0;
     //TRISF = 0;

   	 // Signal WR_BAR to LOW indicate to the PIC18F4550 USB Controller that there is valid data
 	 // on the parallel bus that needs to be read and processed.  
 	 //WR_BAR = 0;
     SetDigitalOutput(WR_BAR, LOW);

     // Test the 8-Bit parallel port outputs
     PrintToScreen("\r\n Testing Vex Parallel Interface... \r\n"); 
     while(1)
     {
	    //SetDirection(1, OUTPUT);             			// Direction for left stepper motor
        IO1 = IO2 = IO3 = IO4 = IO5 = IO6 = IO7 = IO8 = OUTPUT;

		for(i=0; i<10; i++)
		{
            PrintToScreen("Testing VEX PORT BIT 1 = HIGH \r\n"); 
  	        //SetDigitalOutput(1, HIGH);
            rc_dig_out01 = HIGH;
            rc_dig_out02 = HIGH;
            rc_dig_out03 = HIGH;
            rc_dig_out04 = HIGH;
            rc_dig_out05 = HIGH;
            rc_dig_out06 = HIGH;
            rc_dig_out07 = HIGH;
            rc_dig_out08 = HIGH;

            Wait(1000);
            PrintToScreen("Testing VEX PORT BIT 1 = LOW \r\n"); 
  	        //SetDigitalOutput(1, LOW);
            rc_dig_out01 = LOW;
            rc_dig_out02 = LOW;
            rc_dig_out03 = LOW;
            rc_dig_out04 = LOW;
            rc_dig_out05 = LOW;
            rc_dig_out06 = LOW;
            rc_dig_out07 = LOW;
            rc_dig_out08 = LOW;

      }

		for(i=0; i<10; i++)
		{
			// Test Bit 0 (LSB)
            PrintToScreen("Testing Bit 0 = HIGH \r\n"); 
            LATAbits.LATA0 = HIGH;
            Wait(1000);
            PrintToScreen("Testing Bit 0 = LOW \r\n"); 
            LATAbits.LATA0 = LOW;
            Wait(1000);
        }

		for(i=0; i<10; i++)
		{
			// Test Bit 1 (LSB)
            PrintToScreen("Testing Bit 1 = HIGH \r\n"); 
			LATAbits.LATA1 = HIGH;
            Wait(1000);
            PrintToScreen("Testing Bit 1 = LOW \r\n"); 
			LATAbits.LATA1 = LOW;
            Wait(1000);
        }

		for(i=0; i<10; i++)
		{
			// Test Bit 2 
            PrintToScreen("Testing Bit 2 = HIGH \r\n"); 
			LATAbits.LATA2 = HIGH; 
            Wait(1000);
            PrintToScreen("Testing Bit 2 = LOW \r\n"); 
			LATAbits.LATA2 = LOW; 
            Wait(1000);
        }

		for(i=0; i<10; i++)
		{
			// Test Bit 3 
            PrintToScreen("Testing Bit 3 = HIGH \r\n"); 
			LATAbits.LATA3 = HIGH;
            Wait(1000);
            PrintToScreen("Testing Bit 3 = LOW \r\n"); 
			LATAbits.LATA3 = LOW;
            Wait(1000);
        }

		for(i=0; i<10; i++)
		{
			// Test Bit 4
            PrintToScreen("Testing Bit 4 = HIGH \r\n"); 
			LATAbits.LATA5 = HIGH;
            Wait(1000);
            PrintToScreen("Testing Bit 4 = LOW \r\n"); 
			LATAbits.LATA5 = LOW;
            Wait(1000);
        }

		for(i=0; i<10; i++)
		{
			// Test Bit 5
            PrintToScreen("Testing Bit 5 = HIGH \r\n"); 
			LATFbits.LATF0 = HIGH;
            Wait(1000);
            PrintToScreen("Testing Bit 5 = LOW \r\n"); 
			LATFbits.LATF0 = LOW;
            Wait(1000);
        }

		for(i=0; i<10; i++)
		{
			// Test Bit 6
            PrintToScreen("Testing Bit 6 = HIGH \r\n"); 
			LATFbits.LATF1 = HIGH;
            Wait(1000);
            PrintToScreen("Testing Bit 6 = LOW \r\n"); 
			LATFbits.LATF1 = LOW;
            Wait(1000);
        }

		for(i=0; i<10; i++)
		{
			// Test Bit 7 (MSB)
            PrintToScreen("Testing Bit 7 = HIGH \r\n"); 
			LATFbits.LATF2 = HIGH;
            Wait(1000);
            PrintToScreen("Testing Bit 7 = LOW \r\n"); 
			LATFbits.LATF2 = LOW;
            Wait(1000);
        }
     }
#endif
                                                     // variables. 
    PrintToScreen("Testing Write_Byte... \r\n"); 
    Wait(2000);
 
    // Test the parallel port Write function here.
    while (1)
    {
		for (i=0; i<256; i++)
		{
        	Data_Byte = (byte) i;
        	//PrintToScreen("Write Data Byte is %04X\r\n", i); 
        	PrintToScreen("Write Data Byte is %d\r\n", i); 
        	Write_Byte(Data_Byte);

       		 // Wait a bit
       		 //Wait(5000); 
      		 Wait(500); 
		}
    }     
  


#ifdef TEST
    PrintToScreen("Testing Read_Byte... \r\n"); 
    Wait(2000);

    // Test the parallel port Read function here.
    while (1)
    {

        Data_Byte  = Read_Byte();
        i = (int) Data_Byte;
        //PrintToScreen("Data Byte Read is %04X\r\n", i); 
        PrintToScreen("Data Byte Read is %d\r\n", i); 

        // Wait a bit
        Wait(500); 
    }     

    //Test parallel port control signals.  Works!!!
    //SetupTimers();  
    PrintToScreen("\r\n Initializing the Vex Parallel Interface... \r\n"); 
    
    for (i=0; i<100; i++)
    {
       PrintToScreen("This is a test %d times \r\n", i); 
       //Wait(100};
    }

     // Start with low levels 
	SetDigitalOutput(RD_BAR, LOW);
	SetDigitalOutput(WR_BAR, LOW);
	SetDigitalOutput(CS_BAR, LOW);

    while (1)
    {

     // send RD, WR and CS commands to the dsPIC30F6014 coprocessor 
     // Start with high levels 
     //PrintToScreen("Sending HIGH\r\n"); 
 	 SetDigitalOutput(RD_BAR, HIGH);
	 //SetDigitalOutput(WR_BAR, HIGH);
	 //SetDigitalOutput(CS_BAR, HIGH);

     // Wait a bit
     //for(i=0; i<1000; i++);
     for(i=0; i<10000; i++);
     //for(i=0; i<20000; i++);
     // Wait(500); 


     // Send low levels
     //PrintToScreen("Sending LOW\r\n"); 
 	 SetDigitalOutput(RD_BAR, LOW);
	 //SetDigitalOutput(WR_BAR, LOW);
	 //SetDigitalOutput(CS_BAR, LOW);

     // Wait a bit
     //for(i=0; i<1000; i++);
     for(i=0; i<10000; i++);
     //for(i=0; i<20000; i++);
     // Wait(500); 


    }     
                                                     // variables. 
   Wait(3000);
   PrintToScreen("\r\n Initializing the Vex Parallel Interface... \r\n"); 
#endif

/*
// Set digital output pins and test them

void SetDigitalOutput(unsigned char port, unsigned char value);
unsigned char GetDigitalInput(unsigned char port);
void SetDirection(unsigned char port, unsigned char direction);
*/
	 //SetDirection(9, INPUT);

	// Check RD_BAR signal. Works!!! 
 
 
	while(1)
	{
		// Toggle RD_BAR signal
	//	RD_BAR = ~RD_BAR;
		// Read the BUSY_BAR signal from the co-processor
		//PrintToScreen("RD_BAR = %d \r", PORTFbits.RF5);
		Wait(500);
		//Wait(1);
	}

	// Read a ramp 0..255 from the PIC18F4550 USB Controller using the 8-Bit Parallel Interface
	while (1)
	{
 
			// Write a byte to 8-Bit parallel interface so that it can be read by the PIC18F4550 USB Controller
			Data_Byte = Read_Byte();
			PrintToScreen("Data_Byte read = %d \r", Data_Byte);

			// Wait a bit
			Wait(500);

	}



	// Check 8 Data Bits.  
 
	Data_Byte = 0xAA;
	while(1)
	{
		// Toggle WR_BAR signal
		Write_Byte(Data_Byte);

        // Complement the bits
        //Data_Byte = ~Data_Byte;
		// Read the BUSY_BAR signal from the co-processor
		//PrintToScreen("RD_BAR = %d \r", PORTFbits.RF5);
		//Wait(200);
		Wait(1);
	}\

 
	// Check WR_BAR signal.  Works!!! 
 
     //WR_BAR_TRIS = 0;        // Set RD_BAR pin direction to output       

	while(1)
	{
		// Toggle WR_BAR signal
	//	WR_BAR = ~WR_BAR;
		// Read the BUSY_BAR signal from the co-processor
		//PrintToScreen("RD_BAR = %d \r", PORTFbits.RF5);
		//Wait(200);
		Wait(1);
	}



  
	// Check ready line  Workls!!!
 	 //SetDirection(9, INPUT);

	while(1)
	{
		// Read the BUSY_BAR signal from the co-processor
		PrintToScreen("BUSY_BAR = %d \r", PORTFbits.RF5);
		Wait(200);
	}



	// Send a ramp 0..255 to the PIC18F4550 USB Controller using the 8-Bit Parallel Interface
	while (1)
	{
		for (i=0; i<256; i++)
		{
			// Write a byte to 8-Bit parallel interface so that it can be read by the PIC18F4550 USB Controller
			PrintToScreen("Data_Byte written = %d \r", i);
			Write_Byte((byte) i);

			// Wait a bit
			//Wait(500);
		}
	}



	// This code works!!!
  	TwoWheelDrive(1, 2); // define robot two wheels 

	while (1)
	{
		// Get data from first joystick (right)  Works!!!
  		Joy_X_1 = GetOIAInput(PORT_2, CHANNEL_1);
		Joy_Y_1 = GetOIAInput(PORT_2, CHANNEL_2);

		// Get data from second joystick (left)  Works!!!
   		Joy_X_2 = GetOIAInput(PORT_2, CHANNEL_3);
		Joy_Y_2 = GetOIAInput(PORT_2, CHANNEL_4);

    	PrintToScreen("Joy_X_1 = %d, Joy_Y_1 = %d, Joy_X_2 = %d, Joy_Y_2 = %d \r", Joy_X_1, Joy_Y_1, Joy_X_2, Joy_Y_2 ); 
		//Wait(500);


		// Move the selected motors.  Works!!!
		SetPWM (1,Joy_X_1);
		SetPWM (2,Joy_Y_1);		
        SetPWM (3,Joy_X_2);
		SetPWM (4,Joy_Y_2);

 /*
		// Move motors using updated joytick Arcade style positions 
		Arcade2(PORT_1,
                CHANNEL_1, CHANNEL_2,
                LeftPWM, RightPWM,
                0, 0);
*/

	}
/*
An earlier search didn't turn up anything on SetMotor() or SetServo() but a second search turned up some examples using SetPWM(). 
I believe my problem has to do with the size of the variable I was using as an argument. Things seem to be working if I use:

unsigned char armASpeed = 0;
while(1) {
armASpeed = GetRxInput(1,3);
PrintToScreen("armASpeed: %d\n",(int)armASpeed);
SetPWM(8, armASpeed);
}

*/
	while(1)
	{
		// Read Vex RC joystick positions 
 		OIToPWM(PORT_1, 1,  LeftPWM , 0);
	    OIToPWM(PORT_1, 2,  RightPWM, 0);
    	PrintToScreen("LeftPWM = %d, RightPWM = %d \r", LeftPWM, RightPWM); 

		// Move motors using updated joytick Arcade style positions 
		Arcade2(PORT_1,
                CHANNEL_1, CHANNEL_2,
                LeftPWM, RightPWM,
                0, 0);
	}


	TwoWheelDrive(1, 2); // define robot two wheels 
	StartUltrasonic(1, 7); // start ultrasonic rangefinder 
	while (GetUltrasonic(1, 7) > 24) // loop until about 2 feet away 
	{ 
		Drive(100, 0); // drive forward } 
		StopUltrasonic(1, 7); // turn off rangefinder 
		Drive(0, 0); // stop driving
	 }
}
