/*******************************************************************************
* FILE NAME: user_routines_fast_DDT.c <VEX VERSION>
*
* DESCRIPTION:
*  This is a modified version of the Default Code.  It includes additions
*  to allow it to be used with the Dynamic Debug Tool and the PicSerialDrv.*
*  files.
*  This file is where the user can add their custom code within the framework
*  of the routines below. 
*
* USAGE:
*  You can either modify this file to fit your needs, or remove it from your 
*  project and replace it with a modified copy. 
*
* OPTIONS:  Interrupts are disabled and not used by default.
*
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines_parallel.h"
#include "PicSerialdrv.h"
#include "printf_lib.h"
#include "delays.h"

// Declare global types here

#define ENABLE_AUTONOMOUS_MODE   0
#define PORT_A  0
#define PORT_B	1
#define PORT_C	2
#define PORT_D	3
#define PORT_E	4
#define PORT_F	5
#define PORT_G	6
#define PORT_H	7
#define PORT_J 	8


// Timer Interrupt Macros
#define TIMER0_INTERRUPT_ENABLE        	INTCONbits.TMR0IE       // Timer 0 Interrupt Enable
#define TIMER1_INTERRUPT_ENABLE        	PIE1bits.TMR1IE       // Timer 1 Interrupt Enable
#define TIMER2_INTERRUPT_ENABLE        	PIE1bits.TMR2IE       // Timer 2 Interrupt Enable

#define TIMER0_INTERRUPT_FLAG       	INTCONbits.TMR0IF 		// Timer 0 Interrupt Flag
#define TIMER1_INTERRUPT_FLAG       	PIR1bits.TMR1IF 		// Timer 1 Interrupt Flag
#define TIMER2_INTERRUPT_FLAG       	PIR1bits.TMR2IF 		// Timer 2 Interrupt Flag

#define TIMER0_INTERRUPT_PRIORITY       INTCON2bits.TMR0IP       // Set timer interrupt 0 priority to high
#define TIMER1_INTERRUPT_PRIORITY       IPR1bits.TMR1IP          // Set timer interrupt 1 priority to high
#define TIMER2_INTERRUPT_PRIORITY       IPR1bits.TMR2IP          // Set timer interrupt 2 priority to high
 
// Parallel Port Interrupt Macros
#define CS_INTERRUPT_ENABLE        		INTCONbits.INT0IE       // CS_BAR Interrupt Enable
#define WRITE_INTERRUPT_ENABLE       	INTCON3bits.INT2IE      // WR_BAR Interrupt Enable - The INT2 pin is RB2/INTERRUPTS 1. 
#define READ_INTERRUPT_ENABLE       	INTCON3bits.INT3IE      // RD_BAR Interrupt Enable - The INT3 pin is RB3/INTERRUPTS 2.

#define CS_INTERRUPT_FLAG        		INTCONbits.INT0IF		// CS_BAR Interrupt Flag
#define WRITE_INTERRUPT_FLAG       		INTCON3bits.INT2IF		// WR_BAR Interrupt Flag
#define READ_INTERRUPT_FLAG        		INTCON3bits.INT3IF		// RD_BAR Interrupt Flag

#define CS_INPUT_EDGE_DETECTION   		INTCON2bits.INTEDG0		// CS_BAR Input Edge Detection
#define READ_INPUT_EDGE_DETECTION   	INTCON2bits.INTEDG1		// RD_BAR  Input Edge Detection
#define WRITE_INPUT_EDGE_DETECTION   	INTCON2bits.INTEDG2		// WR_BAR Input Edge Detection
#define READY                            rc_dig_out12  
 
/*
 *-------------------------------------------------------------------------------
 *---------- Aliases for INTERRUPTS connectors (inputs) -------------------------
 *--------------------------------------------------------------------------n -----
 *  Below are aliases for the input pins labelled INTERRUPTS on the VEX Robot Controller.
 *  These pins are connected to PULL-UPS, meaning they are normally HIGH (1).
 */

/*
#define rc_dig_int1     PORTBbits.RB2
#define rc_dig_int2     PORTBbits.RB3
#define rc_dig_int3     PORTBbits.RB4
#define rc_dig_int4     PORTBbits.RB5
#define rc_dig_int5     PORTBbits.RB6
#define rc_dig_int6     PORTBbits.RB7
*/

// Parallel Port Interrupt States
#define  PARALLEL_CLEAR   0
#define  PARALLEL_SET     1

#define TIMER0                  // Use Timer 0 for driving stepper motors
//#define TIMER1                 // Use Timer 1 for driving stepper motors
//#define TIMER2                 // Use Timer 2 for driving stepper motors
//#define TIMER3                // Use Timer 3 for driving stepper motors

/*** DEFINE USER VARIABLES AND INITIALIZE THEM HERE ***/
//enum Port_T {PORT_A, PORT_B, PORT_C, PORT_D, PORT_E, PORT_F, PORT_G, PORT_H, PORT_J}; 
 
// Define data structure for virtual 8-Bit Port.  This can be 
// extended to 16-Bit or 32-Bit virtual ports similar to the BS2/BSX Stamp

typedef volatile union  {
    byte Port_Value;		// The port value as a byte
    struct {
        byte VR_BIT_7:1;
        byte VR_BIT_6:1;
        byte VR_BIT_5:1;
        byte VR_BIT_4:1;
        byte VR_BIT_3:1;
        byte VR_BIT_2:1;
        byte VR_BIT_1:1;
        byte VR_BIT_0:1;
/*
        byte VR_BIT_0:1;
        byte VR_BIT_1:1;
        byte VR_BIT_2:1;
        byte VR_BIT_3:1;
        byte VR_BIT_4:1;
        byte VR_BIT_5:1;
        byte VR_BIT_6:1;
        byte VR_BIT_7:1;  
*/ 
    } ;
 
} VR_8_BIT_PORT_T;   

 
// Declare function headers here

 
  
//#ifdef STDIO_INTERRUPTS

// Include stdio interrupt handlers for USART

//#include "stdio.inc"
//#endif



   

// Declare local function headers here
VR_8_BIT_PORT_T Read_Virtual_Port(byte Port_ID);

VR_8_BIT_PORT_T VR_Port_Data;	// Virtual port data (8-Bits)



word            DataWord = 0;                                   // Data word read
word            DataByte = 0;                                   // Data read from parallel port

 
word            CS_Interrupt_Count0   = 0;                      // Reset CS_BAR Interrupt count
word            Read_Interrupt_Count = 0;                       // Reset RD_BAR Interrupt count
word            Write_Interrupt_Count = 0;                      // Reset WR_BAR Interrupt count
word            Ready_Interrupt_Count = 0;                      // Reset READY  Interrupt count
word            InterruptCount4 = 0;                            // Reset PortInterrupt 4 count
byte Buffer[64]; // Buffer used to hold a block of 64 bytes sent from host

// Declare function headers here
byte Read_Byte(void);
void Write_Byte(byte Data);
void IO_Initialization(void);
// Functions for handling virtual ports
VR_8_BIT_PORT_T Read_Virtual_Port(byte Port_ID);
void Write_Virtual_Port(byte Port_ID, VR_8_BIT_PORT_T The_Data);



/*******************************************************************************
* FUNCTION NAME: InterruptVectorLow
* PURPOSE:       Low priority interrupt vector
* CALLED FROM:   nowhere by default
* ARGUMENTS:     none
* RETURNS:       void
* DO NOT MODIFY OR DELETE THIS FUNCTION 
*******************************************************************************/
#pragma code InterruptVectorLow = LOW_INT_VECTOR
void InterruptVectorLow (void)
{
  _asm
    goto InterruptHandlerLow  /*jump to interrupt routine*/
  _endasm
}


/*******************************************************************************
* FUNCTION NAME: InterruptHandlerLow
* PURPOSE:       Low priority interrupt handler
* If you want to use these external low priority interrupts or any of the
* peripheral interrupts then you must enable them in your initialization
* routine.  Innovation First, Inc. will not provide support for using these
* interrupts, so be careful.  There is great potential for glitchy code if good
* interrupt programming practices are not followed.  Especially read p. 28 of
* the "MPLAB(R) C18 C Compiler User's Guide" for information on context saving.
* CALLED FROM:   this file, InterruptVectorLow routine
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
#pragma code
#pragma interruptlow InterruptHandlerLow save=PROD /* You may want to save additional symbols. */

void InterruptHandlerLow ()     
{                               
  unsigned char int_byte;       
  if (WRITE_INTERRUPT_FLAG )         /* The INT2 pin is RB2/INTERRUPTS 1. */
  { 
  
    // Clear Parallel Port Write interrupt flag
   	WRITE_INTERRUPT_FLAG = PARALLEL_CLEAR;

  	// Call Write interrupt handler
   	Process_Write_Interrupt();


  }
  else if (READ_INTERRUPT_FLAG)    /* The INT3 pin is RB3/INTERRUPTS 2. */
  {
    
   	 // Process the Parallel Port Read interrupt
     // Clear the Parallel Port Read interrupt
  	 READ_INTERRUPT_FLAG = PARALLEL_CLEAR;

    // Call Parallel Port Read interrupt handler
   	Process_Read_Interrupt();


  }
  else if (INTCONbits.RBIF)       /* INTERRUPTS 3-4 (RB4, RB5, RB6, or RB7) changed. */
  {
    int_byte = PORTB;             /* You must read or write to PORTB   */
    INTCONbits.RBIF = 0;          /* and clear the interrupt flag      */
  }   
  else	if (CS_INTERRUPT_FLAG) 
  {
    // Process the Chip Select Parallel Port Interrupt
    // Clear CS interrupt flag
    CS_INTERRUPT_FLAG  = PARALLEL_CLEAR; 

    // Call CS interrupt handler
    //Process_CS_Interrupt();

  }  

  /* to clear the interrupt condition. */
  else     
  {
    CheckUartInts();
  }



#ifdef NEW_CODE  
		// Check for the type of timer interrupt (TMR0IF..TMR3IF) and process it
#ifdef TIMER0
    	if (TIMER0_INTERRUPT_FLAG  == 1)                         // Works!!!
#endif

#ifdef TIMER1
    	if (TIMER1_INTERRUPT_FLAG  == 1)
#endif
    	{
 
     		// count
    		//LED0 = ~LED0;                                // Toggle the timer 0 LED
                                                           // interrupt  LED indicator
#ifdef TIMER0   
    		TIMER0_INTERRUPT_FLAG = 0;                     // Clear the Timer 0 overflow flag
   	
   			// Reload Timer 0 register here
    		WriteTimer0(0x0000);                           // for a 26 milli-second period ( Hz) 
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
	
    		//WriteTimer0(0x6967);                         // 2's complement of 39063 (0x6769) for a 1 second period (1 Hz) 
                                                           // using timer 0 and a 1:128 pre-scale.
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
  
#endif

#ifndef TIMER1
       	     TIMER1_INTERRUPT_FLAG = 0;                     // Clear the Timer 0 overflow flag
     		 //WriteTimer1(0x0000);                           // for a 26 milli-second period ( Hz) 
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
             WriteTimer1(0x8FFD);                                   // 2's complement of 625 (0xFD8F) for a 1 milli-second delay
                                                           // (Use LSB..MSB order to compensate for Microchip 
                                                           // timer library bug)  
                                                           // (Use "WriteTimer1_16" instead of "WriteTimer1" to compensate for another Microchip 

#endif

 			// Toggle each stepper motor and update their counts 
			if (StepperCount0 > 0)  
			{
        		// Send commands to  Stepper Motor #0
   				Step_0 = ~Step_0;

        		// Decrement count to Stepper Motor #0
				StepperCount0--;  
			} 

 			if (StepperCount1 > 0)  
			{
        		// Send commands to  Stepper Motor #1
  				Step_1  = ~Step_1;

        		// Decrement count to Stepper Motor #1
				StepperCount1--;  
			}




 			if (StepperCount2 > 0)
			{  
       			 // Send commands to  Stepper Motor #2
   				Step_2 = ~Step_2;

        		// Decrement count to Stepper Motor #2
				StepperCount2--;   

			}

  			if (StepperCount3 > 0)  
			{

        		// Send commands to  Stepper Motor #3
   				Step_3 = ~Step_3;

        		// Decrement count to Stepper Motor #3
				StepperCount3--;   
			}

 			if (StepperCount4 > 0)  
			{
        		// Send commands to  Stepper Motor #4
   				Step_4 = ~Step_4;

        		// Decrement count to Stepper Motor #4
				StepperCount4--;   
			}

 			if (StepperCount5 > 0)  
			{
        		// Send commands to  Stepper Motor #5
   				Step_5 = ~Step_5;

        		// Decrement count to Stepper Motor #5
				StepperCount5--;   
			}

			if (StepperCount6 > 0)  
			{
        		// Send commands to  Stepper Motor #6
   				Step_6 = ~Step_6;

        		// Decrement count to Stepper Motor #6
				StepperCount6--;   
			}

 			if (StepperCount7 > 0)  
			{
        		// Send commands to  Stepper Motor #7
   				Step_7 = ~Step_7;

        		// Decrement count to Stepper Motor #7
				StepperCount7--;   
			}
#endif





}	//This return will be a "retfie fast", since this is in a #pragma interrupt section 





void User_Autonomous_Code(void)
{
}

/*******************************************************************************
* FUNCTION NAME: Process_Data_From_Local_IO
* PURPOSE:       Execute user's realtime code.
* You should modify this routine by adding code which you wish to run fast.
* It will be executed every program loop, and not wait for fresh data 
* from the master processor.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Process_Data_From_Local_IO(void)
{
  /* Add code here that you want to be executed every program loop. */

//	SetPWM(1, 150);

    Process_Debug_Stream();
}


/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/*******************************************************************************
* FUNCTION:    Initialize_Parallel_Port ()
* PURPOSE:     Initializes the VEX Parallel Port.
* CALLED FROM: Enable_Sensors() in jvex_utilities.c
* PARAMETERS:  None
* RETURNS:     Nothing
* COMMENTS:    Place "#include "jvex_encoders.h" in the includes section
*              of jvex_utilities.c then call Initialize_Encoders() in
*              Enable_Sensors().
*******************************************************************************/
void Initialize_Parallel_Port ( void )
{

  int j; 

  // Disable the Parallel Port interrupts
  CS_INTERRUPT_ENABLE 	 = PARALLEL_CLEAR;	// Disable the CS_BAR Interrupt Flag
  READ_INTERRUPT_ENABLE	 = PARALLEL_CLEAR;	// Disable the CS_BAR Interrupt Flag
  WRITE_INTERRUPT_ENABLE = PARALLEL_CLEAR;	// Disable the CS_BAR Interrupt Flag

  // Zero Parallel Port interrupt counts
  CS_Interrupt_Count0   = 0;                      // Reset CS_BAR Interrupt count
  Read_Interrupt_Count = 0;                      // Reset RD_BAR Interrupt count
  Write_Interrupt_Count = 0;                      // Reset WR_BAR Interrupt count
  Ready_Interrupt_Count = 0;                      // Reset READY  Interrupt count

  // Set the Parallel Port interrupts to detect the rising edge
  CS_INPUT_EDGE_DETECTION    = PARALLEL_SET;
  READ_INPUT_EDGE_DETECTION  = PARALLEL_SET;
  WRITE_INPUT_EDGE_DETECTION = PARALLEL_SET;

  // Reset the Parallel Port interrupt flags
  CS_INTERRUPT_FLAG    		= PARALLEL_CLEAR;     	// Clear the CS_BAR Interrupt Flag
  READ_INTERRUPT_FLAG     	= PARALLEL_CLEAR;		// Clear the RD_BAR Interrupt Flag
  WRITE_INTERRUPT_FLAG      = PARALLEL_CLEAR;		// Clear the WR_BAR Interrupt Flag

  // Enable the encoder interrupts
  CS_INTERRUPT_ENABLE 	 = PARALLEL_SET;	// Enable the CS_BAR Interrupt Flag
  READ_INTERRUPT_ENABLE	 = PARALLEL_SET;	// Enable  the CS_BAR Interrupt Flag
  WRITE_INTERRUPT_ENABLE = PARALLEL_SET;	// Enable  the CS_BAR Interrupt Flag

  // Initialize block of 64 bytes used to transfer messages between th VEX Controller and the host
  for (j=0; j<64; j++)
  {
    Buffer[j] = j;
  }
 
}

/*******************************************************************************
* FUNCTION:    Disable_Parallel_Port()
* PURPOSE:     Disables the VEX Parallel Port interrupts and event processing.
* CALLED FROM: Disable_Sensors() in jvex_utilities.c
* PARAMETERS:  None
* RETURNS:     Nothing  
* COMMENTS:    Place "#include "jvex_encoders.h" in the includes section
*              of jvex_utilities.c then call Disable_Encoders() in
*              Disable_Sensors().
*******************************************************************************/
void Disable_Parallel_Port ( void )
{
  // Disable the Parallel Port interrupts
  CS_INTERRUPT_ENABLE 	 = PARALLEL_CLEAR;	// Disable the CS_BAR Interrupt Flag
  READ_INTERRUPT_ENABLE	 = PARALLEL_CLEAR;	// Disable the CS_BAR Interrupt Flag
  WRITE_INTERRUPT_ENABLE = PARALLEL_CLEAR;	// Disable the CS_BAR Interrupt Flag


  // Reset the Parallel Port interrupt flags
  CS_INTERRUPT_FLAG    		= PARALLEL_CLEAR;     	// Clear the CS_BAR Interrupt Flag
  READ_INTERRUPT_FLAG     	= PARALLEL_CLEAR;		// Clear the RD_BAR Interrupt Flag
  WRITE_INTERRUPT_FLAG      = PARALLEL_CLEAR;		// Clear the WR_BAR Interrupt Flag

   // Zero Parallel Port interrupt counts
  CS_Interrupt_Count0   = 0;                      // Reset CS_BAR Interrupt count
  Read_Interrupt_Count = 0;                      // Reset RD_BAR Interrupt count
  Write_Interrupt_Count = 0;                      // Reset WR_BAR Interrupt count
  Ready_Interrupt_Count = 0;                      // Reset READY  Interrupt count

}
 
//********************************************************************************
//* Process_Read_Interrupt - Handles Parallel Port interrupt 1 - RD_BAR - Read data byte from the dsPIC30F6014
//********************************************************************************

void  Process_Read_Interrupt(void)
{
    int j;
#ifndef NEW_CODE
    //BUSY_BAR_TRIS = 1;    	// Set BUSY_BAR pin direction to input         

    READY =  1;                                              // Co-processor is busy
	// RD_BAR - Read data byte from the dsPIC30F6014
    Read_Interrupt_Count++;                                       // Increment RD_BAR interrupt count 
    //LED1 = ON;                                             // Toggle the interrupt 1 LED 2 indicator
   	//LED_1 = ~LED_1;                                        // Toggle the interrupt 1 LED 2 indicator
    rc_dig_out15   = ~rc_dig_out15; // LED 15   
 
    // Set port D to output
    //TRISD = 0x0000;
    //TRISD = 0x00;


    // Get the data byte from the VEX Controller and send it back to the host using the Virtual Port

#ifdef SEND_BYTE
    // Test Code - Return the current interrupt count for now as the data byte (0:255)
    DataByte = Read_Interrupt_Count % 256;
    Write_Byte(DataByte);  
  	printf("Read Data Byte is %d\r\n", DataByte); 
#endif


    // Test Code - send ramp 0..63 bytes to host 
    for (j=0; j<64; j++)
    {
       Write_Byte(Buffer[j]);  
    }
  	//printf("Read Data Byte is %d\r\n", DataByte); 

    //DataWord = (DataByte << 8);
    //PORTD = DataWord;
    //LATD = DataWord;
    //LATD = DataByte;

    
#ifdef DEBUG
    if(mUSBUSARTIsTxTrfReady())
    {
       putrsUSBUSART("RD_BAR  \r\n");
       stringPrinted = TRUE;
    }
#endif
    // Lcd_Printf("RD_BAR \n");
    READY = 0;                                               // Co-processor is ready
#endif
}

//********************************************************************************
//* Process_Write_Interrupt - Handles Parallel Port interrupt 2 - WR_BAR - Write data byte to the dsPIC30F6014
//********************************************************************************

void  Process_Write_Interrupt(void)
{
    int j;
#ifndef NEW_CODE
    //BUSY_BAR_TRIS = 1;    	// Set BUSY_BAR pin direction to input         
    READY =  1;                                              // Co-processor is busy

	// WR_BAR - Write data byte to the dsPIC30F6014
    Write_Interrupt_Count++;                                // Increment WR_BAR interrupt    
    //LED2 = ON;                                            // Toggle the port  interrupt2 LED indicator
    //LED_2 = ~LED_2;                                       // Toggle the port  interrupt2 LED indicator
    rc_dig_out16   = ~rc_dig_out16; // LED 16  

    // Set the direction of Port D bits 8:15 to input to read 8-Bit data on the parallel bus
	//TRISD = 0xFFFF;
	//TRISD = 0xFF;
#ifdef RECEIVE_BYTE 
   	// Read the data from the host using the Virtual Port
	DataByte = Read_Byte();  
  	printf("Write Data Byte is %d\r\n", DataByte); 
#endif

    // Read a block of 64 bytes sent from host
    for(j=0; j<64; j++)
    {
        Buffer[j] = Read_Byte();
    }

    //DataByte = PORTD;

    //DataWord = PORTD;
    //DataByte =  (DataWord >> 8);
 
    // Set port D to output
    //TRISD = 0x0000;
    //Lcd_Printf("WD_BAR \n");
	//Lcd_Set_Character_Position(0, 1);
    //Lcd_Printf("DataByte = %d \n", DataByte);
    //printf("DataByte = %d \r\n", DataByte);
    //pause(1000);
#ifdef DEBUG
    if(mUSBUSARTIsTxTrfReady())
    {
       putrsUSBUSART("WD_BAR  \r\n");
       stringPrinted = TRUE;
    }
#endif
    READY = 0;                                               // Co-processor is ready
#endif
}

//********************************************************************************
//* ProcessInterrupt3 - Handles Parallel Port interrupt 3 - CS_BAR - Chip select the dsPIC30F6014
//* in order to initialize it.
//********************************************************************************

void   Process_CS_Interrupt(void)
{
#ifndef NEW_CODE
 	// CS_BAR - Chip select the dsPIC30F6014
    CS_Interrupt_Count0++;                                         // Increment CS_BAR interrupt  
    READY = 0;

    // Set port D to output
    //TRISD = 0x0000;
    TRISD = 0x00;
    // LED_3 = ~LED_3;                                              // Toggle the port interrupt 3 LED indicator
    //Lcd_Printf("CS_BAR \n");

/*
    if(mUSBUSARTIsTxTrfReady())
    {
       putrsUSBUSART("CS_BAR  \r\n");
       stringPrinted = TRUE;
    }
*/
#endif
}

//********************************************************************************
//* ProcessInterrupt4 - Processes Portbit 7.
//********************************************************************************

void            ProcessInterrupt4(void)
{
#ifdef NEW_CODE
    InterruptCount4++;                                        // Increment port interrupt 4 count
    // Set port D to output
    //TRISD = 0x0000;

     //LED_4 = ~LED_4;                                             // Toggle the port interrupt 4 LED indicator
    //Lcd_Printf("ProcessInterrupt4\n");
#endif
}

//******************************************************************************
//* Read_Virtual_Port - Read a byte of data from the selected Port,  
//* Port_ID range (PORT_A..PORTE).  It reads the bits from the
//* assigned ports and saves them to Virtual Port A, bits A(0:5), Port F bits(0:2)  
//*   
//******************************************************************************

VR_8_BIT_PORT_T Read_Virtual_Port(byte Port_ID)
{
	VR_8_BIT_PORT_T The_Virtual_Port;

	switch(Port_ID)
	{
		case PORT_A:
		{
/*
			// Set PORT A bits to inputs
			TRISAbits.TRISA0 = 1;
			TRISAbits.TRISA1 = 1;
			TRISAbits.TRISA2 = 1;
			TRISAbits.TRISA3 = 1;
			TRISAbits.TRISA5 = 1;

			// Set PORT F bits to inputs
			TRISFbits.TRISF0 = 1;
			TRISFbits.TRISF1 = 1;
			TRISFbits.TRISF2 = 1;
*/
            //DefineControllerIO (   0, INPUT ,INPUT ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
            IO1 = IO2 = IO3 = IO4 = IO5 = IO6 = IO7 = IO8 = INPUT;

            The_Virtual_Port.VR_BIT_0 = PORTAbits.RA0;
		 	The_Virtual_Port.VR_BIT_1 = PORTAbits.RA1;;
            The_Virtual_Port.VR_BIT_2 = PORTAbits.RA2;
		 	The_Virtual_Port.VR_BIT_3 = PORTAbits.RA3;
            The_Virtual_Port.VR_BIT_4 = PORTAbits.RA5;;
		 	The_Virtual_Port.VR_BIT_5 = PORTFbits.RF0;
		 	The_Virtual_Port.VR_BIT_6 = PORTFbits.RF1;
		 	The_Virtual_Port.VR_BIT_7 = PORTFbits.RF2;
			break;
		}

		case PORT_D:
		{
			The_Virtual_Port.Port_Value = PORTD;
			break;
		}

	}
	return The_Virtual_Port;	
}


//******************************************************************************
//* Write_Virtual_Port - Write a byte of data from the selected port, 
//* Port_ID range (PORT_A..PORTE). It Distributes the bits from the virtual data byte to the
//* Port_ID range (PORT_A..PORTE) using the following
//* assigned ports and saves them to Virtual Port A, bits A(0:5), Port F bits(0:2)  
//******************************************************************************
void Write_Virtual_Port(byte Port_ID, VR_8_BIT_PORT_T The_Data)
{
	// Distribute the bits from the virtual data byte to the
	// assigned for Virtual Port A, bits to Virtual Port A, bits A(0:5), Port F bits(0:2) 
	switch(Port_ID)
	{
		case PORT_A:
		{
/*
			// Set PORT A bits to outputs
			TRISAbits.TRISA0 = 0;
			TRISAbits.TRISA1 = 0;
			TRISAbits.TRISA2 = 0;
			TRISAbits.TRISA3 = 0;
			TRISAbits.TRISA5 = 0;

			// Set PORT F bits to outputs
			TRISFbits.TRISF0 = 0;
			TRISFbits.TRISF1 = 0;
			TRISFbits.TRISF2 = 0;
*/
            //DefineControllerIO (   0, OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
            IO1 = IO2 = IO3 = IO4 = IO5 = IO6 = IO7 = IO8 = OUTPUT;

			// Copy the virtual port bits to the physical port bit assignments 
            LATAbits.LATA0 = The_Data.VR_BIT_0;
			LATAbits.LATA1 = The_Data.VR_BIT_1;
			LATAbits.LATA2 = The_Data.VR_BIT_2;
			LATAbits.LATA3 = The_Data.VR_BIT_3;
			LATAbits.LATA5 = The_Data.VR_BIT_4;
			LATFbits.LATF0 = The_Data.VR_BIT_5;
			LATFbits.LATF1 = The_Data.VR_BIT_6;
			LATFbits.LATF2 = The_Data.VR_BIT_7;
			break;
		}
 
		case PORT_D:
		{
			// Set PORT D bits to outputs
			TRISD = 0;

			PORTD = The_Data.Port_Value;

			break;
		}

	}
}

//******************************************************************************************
//* Read_Byte -  This routine reads a byte of data from the 8-Bit parallel interface into the 
//* Vex Controller.
//******************************************************************************************
byte Read_Byte(void)
{
	byte Data_Byte = 0;
	//printf("Entering Read_Byte...\r");
    //BUSY_BAR_TRIS = 1;    	// Set BUSY_BAR pin direction to input         
    //RD_BAR_TRIS = 0;        // Set RD_BAR pin direction to output       
   
  	//printf("Trace 1\r");

	// Check to make sure the PIC18F4550 USB Controller is not busy
   	//while(!BUSY_BAR);

	// Copy the Data byte to the 8-Bit parallel port

	// Set signal RD_BAR to LOW indicating to the PIC18F4550 USB Controller that it can put a byte of data
	// on the parallel bus so that it can be read in by the Vex Controller.   
	//RD_BAR = 0;
    //SetDigitalOutput(RD_BAR, LOW);
    //RD_BAR = LOW;

	//printf("Trace 2\r");

	// Check to make sure the PIC18F4550 USB Controller is not busy
	//while(!BUSY_BAR);
 	//printf("Trace 3\r");

	// Read the data from the 8-Bit Parallel Port
	VR_Port_Data = Read_Virtual_Port(PORT_A);
	//printf("Leaving Read_Byte.\r");
   
	// Set signal RD_BAR to HIGH indicating to the PIC18F4550 USB Controller that it can put a byte of data
	// on the parallel bus so that it can be read in by the Vex Controller.   
	//RD_BAR = 1;
	//SetDigitalOutput(RD_BAR, HIGH);
	//RD_BAR = HIGH;

	// Return the data byte
	return VR_Port_Data.Port_Value;
}


//******************************************************************************************
//* Write_Byte -  This routine writes a byte of data to the 8-Bit parallel interface so that
//* it can be read and processed by the PIC18F4550 USB Controller
//******************************************************************************************
 void Write_Byte(byte Data)
{ 
	//printf("Entering Write_Byte...\r");
   
    //BUSY_BAR_TRIS = 1;    	// Set BUSY_BAR pin direction to input         
    //WR_BAR_TRIS = 0;        // Set RD_BAR pin direction to output       

  	//printf("Trace 1\r");
	
	//printf("Data_Byte read = %d \r", BUSY_BAR);
	//printf("Data_Byte read = %d \r", PORTFbits.RF5);

	// Check to make sure the PIC18F4550 USB Controller is not busy
	//while(!BUSY_BAR);
//	{
		//printf("BUSY_BAR = %d \r", (int) BUSY_BAR);
//		Wait(200);
//	}
   //printf("Trace 2\r");

	// Signal WR_BAR to LOW indicate to the PIC18F4550 USB Controller that there is valid data
	// on the parallel bus that needs to be read and processed.  
	//WR_BAR = 0;
    //SetDigitalOutput(WR_BAR, LOW);
    //WR_BAR = LOW;

	// Copy the Data byte to the 8-Bit parallel port
 	VR_Port_Data.Port_Value = Data;
	Write_Virtual_Port(PORT_A, VR_Port_Data);
 

 	//printf("Trace 3\r");

	// Check to make sure the PIC18F4550 USB Controller is not busy
   // while(!BUSY_BAR);
	//{
	//	printf("BUSY_BAR = %d \r", (int) BUSY_BAR);
//		Wait(200);
//	};

	// Signal WR_BAR to HIGH indicate to the PIC18F4550 USB Controller that there is valid data
	// on the parallel bus that needs to be read and processed.  
	//WR_BAR = 0;
    //SetDigitalOutput(WR_BAR, HIGH);
    //WR_BAR =  HIGH;

	//printf("Leaving Write_Byte\r");

}


