/*******************************************************************************
* FILE NAME: user_routines.h
*
* DESCRIPTION: 
*  This is the include file which corresponds to user_routines.c and
*  user_routines_fast.c
*  It contains some aliases and function prototypes used in those files.
*
* USAGE:
*  If you add your own routines to those files, this is a good place to add
*  your custom macros (aliases), type definitions, and function prototypes.
*******************************************************************************/

#ifndef __user_program_h_
#define __user_program_h_

#include "my_types.h"

/*******************************************************************************
                            MACRO DECLARATIONS
*******************************************************************************/
/* Add your macros (aliases and constants) here.                              */
/* Do not edit the ones in ifi_aliases.h                                      */
/* Macros are substituted in at compile time and make your code more readable */
/* as well as making it easy to change a constant value in one place, rather  */
/* than at every place it is used in your code.                               */
/*
 EXAMPLE CONSTANTS:
#define MAXIMUM_LOOPS   5
#define THE_ANSWER      42
#define TRUE            1
#define FALSE           0
#define PI_VAL          3.1415

 EXAMPLE ALIASES:
#define LIMIT_SWITCH_1  rc_dig_int1  (Points to another macro in ifi_aliases.h)
#define MAIN_SOLENOID   solenoid1    (Points to another macro in ifi_aliases.h)
*/

/* Used in limit switch routines in user_routines.c */
#define OPEN        1     /* Limit switch is open (input is floating high). */
#define CLOSED      0     /* Limit switch is closed (input connected to ground). */

// Hardware interface with the  Vex Controller (using Vex Controller PIC18F8520 Pin numbering)

#define CS_BAR        rc_dig_out09      // Bits are shifted out this pin # to the MAX7219.
#define WR_BAR        rc_dig_out10      // Data valid on rising edge of this clock (pin RD5).
#define RD_BAR        rc_dig_out11      // This control pin tells the 7219 to transfer data to LEDs.  
#define BUSY_BAR      rc_dig_in12      	// Signal indicating that the peripheral is ready to accept another parallel command
#define BUSY          rc_dig_in12       // BUSY signal indicating the 8-Bit parallel port data is ready to be read 

//#define BUSY        PORTBbits.RB3           // BUSY signal indicating the 8-Bit parallel port data is ready to be read 
//#define BUSY_DIR   	TRISBbits.TRISB3        // BUSY signal indicating the 8-Bit parallel port data is ready to be read  pin direction 


/*******************************************************************************
                            TYPEDEF DECLARATIONS
*******************************************************************************/
/* EXAMPLE DATA STRUCTURE */
/*
typedef struct
{
  unsigned int  NEW_CAPTURE_DATA:1;
  unsigned int  LAST_IN1:1;
  unsigned int  LAST_IN2:1;
  unsigned int  WHEEL_COUNTER_UP:1;
  unsigned int  :4;
  unsigned int wheel_left_counter;
  unsigned int wheel_right_counter;
} user_struct;
*/

extern word            DataWord  ;                                   // Data word read
extern word            DataByte ;                                   // Data read from parallel port

 
extern word            CS_Interrupt_Count0   ;                      // Reset CS_BAR Interrupt count
extern word            Read_Interrupt_Count  ;                       // Reset RD_BAR Interrupt count
extern word            Write_Interrupt_Count ;                      // Reset WR_BAR Interrupt count
extern word            Ready_Interrupt_Count  ;                      // Reset READY  Interrupt count
//extern word            InterruptCount4  ;                            // Reset PortInterrupt 4 count

/*******************************************************************************
                           FUNCTION PROTOTYPES
*******************************************************************************/

/* These routines reside in user_routines.c */
void User_Initialization(void);
void Process_Data_From_Master_uP(void);
void Default_Routine(void);

/* These routines reside in user_routines_fast.c */
void InterruptHandlerLow (void);  /* DO NOT CHANGE! */
void User_Autonomous_Code(void);  /* Only in full-size FRC system. */
void Process_Data_From_Local_IO(void);

//********************************************************************************
// Function Prototypes
//********************************************************************************

void            ServiceInterrupt(void);                    		// Services the current interrupt (INT1..INT4)
void            SetupPortInterrupts(void);                		// Configures port interrupts (INT1..INT4) and CN interrupts  
void            ProcessCnInterrupt(void);              			// Process Change on Notice (CN) interrupt  
void            Process_CS_Interrupt(void);                     // Processes CS interrupt 
void            Process_Read_Interrupt(void);                   // Processes RD_BAR interrupt 1
void            Process_Write_Interrupt(void);                  // Processes WR_BAR interrupt 2
void            ProcessInterrupt3(void);                       	// Processes port interrupt 3
void            ProcessInterrupt4(void);                       	// Processes port interrupt 4
void            Initialize_Timers (void);                       // Initializes the timers and timer interrupts
void 			Initialize_Steppers (void);                     // Initialize the Stepper Motor counters

#endif
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
